"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.backup = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const zlib_1 = __importDefault(require("zlib"));
const moment_1 = __importDefault(require("moment"));
const child_process_1 = require("child_process");
const utils_1 = require("../lib/utils");
const mysql_1 = require("../lib/mysql");
const logger_1 = require("../lib/logger");
const const_1 = require("../lib/const");
/**
 * 备份数据库
 * @param option 备份选项
 */
async function backup(option) {
    let conn;
    try {
        if (!option.backupFileTag)
            option.backupFileTag = const_1.BACKUP_FILE_TAG;
        let dirname = option.backupDir;
        if (!dirname)
            throw new Error(`backup directory is required`);
        //系统检测
        dirname = path_1.default.resolve(process.cwd(), dirname);
        if (!fs_1.default.existsSync(dirname))
            fs_1.default.mkdirSync(dirname, { recursive: true });
        else if (!fs_1.default.statSync(dirname).isDirectory())
            throw new Error(`backup directory ${dirname} is not directory`);
        //连接数据库
        conn = mysql_1.dbu.getConnection(option.databaseConfig);
        conn.use(option.databaseConfig.database);
        //检测是否能够备份
        const dbs = await conn.query('show databases');
        //生成备份名
        let filename = `${(0, moment_1.default)().format('YYYYMMDD-HHmmss')}.bak`;
        if (option.filenameGenerator) {
            const res = await conn.query('select * from _ver order by ver desc limit 1');
            const ver = res[0].ver;
            filename = option.filenameGenerator({
                version: ver,
                dbname: option.databaseConfig.database
            });
        }
        const outfile = path_1.default.join(dirname, filename);
        conn.close();
        conn = undefined;
        //如果数据库不存在则不备份
        if (!dbs.some(dbi => dbi.Database == option.databaseConfig.database))
            logger_1.logger.warn('backup', 'nothing to be done');
        //开始备份数据库
        else {
            await new Promise(resolve => {
                //创建文件输出流
                const ws = fs_1.default.createWriteStream(outfile);
                ws.write(option.backupFileTag);
                const gzip = zlib_1.default.createGzip();
                //开始备份
                const cmd = [
                    'mysqldump --hex-blob',
                    '-x',
                    ...option.databaseConfig.username ? [`-u${option.databaseConfig.username}`] : [],
                    ...option.databaseConfig.password ? [`-p${option.databaseConfig.password}`] : [],
                    `-h"${option.databaseConfig.host || 'localhost'}" -P${option.databaseConfig.port || 3306} "${option.databaseConfig.database}"`,
                ].join(' ');
                const cp = (0, child_process_1.exec)(cmd, { maxBuffer: const_1.MAX_BUFFER });
                cp.stdout.pipe((0, utils_1.createCryptStream)(option.fileEncryptPassword)).pipe(gzip).pipe(ws);
                ws.on('close', () => resolve());
            });
            logger_1.logger.success('backup', `backup database to ${outfile}`);
        }
    }
    catch (err) {
        if (conn)
            conn.close();
        logger_1.logger.error('backup', err.message);
        throw new err;
    }
}
exports.backup = backup;
