"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.restore = void 0;
const zlib_1 = __importDefault(require("zlib"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const child_process_1 = require("child_process");
const const_1 = require("../lib/const");
const logger_1 = require("../lib/logger");
const mysql_1 = require("../lib/mysql");
const utils_1 = require("../lib/utils");
/**
 * 恢复数据库
 * @param option 数据库恢复选项
 */
async function restore(option) {
    let conn;
    try {
        let filename = option.file || (() => {
            if (!option.backupDir)
                return '';
            const [file] = fs_1.default.readdirSync(option.backupDir).reverse();
            if (!file)
                return '';
            return path_1.default.join(option.backupDir, file);
        })();
        if (!filename)
            throw new Error(`backup file is required`);
        const backupFileTag = option.backupFileTag ? Buffer.from(option.backupFileTag) : const_1.BACKUP_FILE_TAG;
        //基本校验
        filename = path_1.default.resolve(process.cwd(), filename);
        if (!fs_1.default.existsSync(filename))
            throw new Error(`file ${filename} not exists`);
        if (!fs_1.default.statSync(filename).isFile())
            throw new Error(`file ${filename} is not a file`);
        //校验是否是备份文件
        const isBakFile = await new Promise(resolve => {
            const is = fs_1.default.createReadStream(filename, { start: 0, end: backupFileTag.length - 1 });
            is.on('data', di => {
                is.close();
                resolve(Buffer.compare(backupFileTag, Buffer.from(di)) == 0);
            });
            is.read(backupFileTag.length);
        });
        if (!isBakFile)
            throw new Error(`file ${filename} is not a backup file`);
        //连接数据库
        conn = mysql_1.dbu.getConnection(option.databaseConfig);
        conn.use(option.databaseConfig.database);
        //数据库重新初始化
        const dbs = await conn.query('show databases');
        if (dbs.some(dbi => dbi.Database == option.databaseConfig.database)) {
            logger_1.logger.info('restore', `drop old database`);
            await conn.exec('drop database ??', option.databaseConfig.database);
        }
        logger_1.logger.info('restore', `create new database ${option.databaseConfig.database}`);
        await conn.exec('create database ?? default character set = ?', option.databaseConfig.database, option.databaseConfig.charset || 'utf8mb4');
        conn.close();
        conn = undefined;
        //开始备份
        await new Promise((resolve, reject) => {
            const cmd = [
                'mysql',
                ...option.databaseConfig.username ? [`-u${option.databaseConfig.username}`] : [],
                ...option.databaseConfig.password ? [`-p${option.databaseConfig.password}`] : [],
                `-h"${option.databaseConfig.host || 'localhost'}" -P${option.databaseConfig.port || 3306} "${option.databaseConfig.database}"`,
            ].join(' ');
            //创建进程
            const cp = (0, child_process_1.exec)(cmd, { maxBuffer: const_1.MAX_BUFFER });
            cp.once('close', code => resolve((code == 0) ? true : false));
            //写入数据流
            const gunzip = zlib_1.default.createGunzip();
            gunzip.on('error', err => {
                cp.kill();
                reject(new Error('gunzip: ' + err.message));
            });
            let load = 0;
            const total = fs_1.default.statSync(filename).size - backupFileTag.length;
            const instream = fs_1.default.createReadStream(filename, { start: backupFileTag.length });
            gunzip.pipe((0, utils_1.createCryptStream)(option.fileEncryptPassword)).pipe(cp.stdin);
            instream.on('data', data => {
                instream.pause();
                load += data.length;
                option.onProgress?.(load, total);
                gunzip.write(data, (err) => {
                    if (err)
                        gunzip.destroy(err);
                    else
                        instream.resume();
                });
            });
            instream.once('end', () => gunzip.end());
        });
        logger_1.logger.clearLine();
        logger_1.logger.success('restore', `database restore successfully`);
    }
    catch (err) {
        logger_1.logger.error('restore', err.message);
        if (conn)
            conn.close();
        throw err;
    }
}
exports.restore = restore;
