"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.update = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const mysql_1 = require("../lib/mysql");
const logger_1 = require("../lib/logger");
//加载版本文件
function loadVersions(versionDir) {
    if (versionDir) {
        fs_1.default.readdirSync(versionDir).forEach(file => {
            const ext = path_1.default.extname(file);
            if (ext == '.js' || ext == '.ts')
                require(path_1.default.join(versionDir, file));
        });
    }
    return mysql_1.dbu.getVersions();
}
/**
 * 开始版本升级
 * @param version 版本号，如果为true表示升级到最新
 */
async function update(option) {
    let conn;
    try {
        //加载版本文件并排序
        const versions = loadVersions(option.versionDir).sort((v1, v2) => {
            if (v1.ver == v2.ver)
                return 0;
            return (v1.ver > v2.ver) ? 1 : -1;
        });
        const vdict = {};
        versions.forEach(vi => {
            vi.ver = parseFloat(vi.ver.toFixed(2));
            if (vi.ver <= 0)
                throw new Error('version cannot <0');
            if (vdict[vi.ver])
                throw new Error(`got same version ${vi.ver} in versions`);
            vdict[vi.ver] = vi.ver;
        });
        if (!versions.length)
            throw new Error('no version found');
        const dest = option.version ? parseFloat(option.version) : versions[versions.length - 1].ver;
        if (!versions.some(v => v.ver == dest))
            throw new Error(`unknown version ${dest}`);
        //创建数据库连接
        const conf = option.databaseConfig;
        conn = mysql_1.dbu.getConnection(option.databaseConfig);
        //数据库初始化
        const dbs = await conn.query('show databases');
        if (!dbs.some(di => di.Database == conf.database)) {
            logger_1.logger.info('update', `initial database [${conf.database}]`);
            await conn.exec('create database ?? default character set = ?', conf.database, conf.charset);
            //创建版本表
            await conn.mktbl(`${conf.database}._ver`)
                .column('ver', 'decimal', { length: '20,2', comment: '版本号' })
                .column('ctime', 'datetime', { comment: '版本创建时间' })
                .primary('ver')
                .index('ctime')
                .done();
        }
        await conn.use(conf.database);
        //获取版本列表
        const dbVersions = await conn.query('select * from _ver order by ver asc').then(res => res.map(vi => vi.ver));
        //获取开始版本
        const [fromVer] = versions.filter(v => !dbVersions.some(dv => dv >= v.ver));
        if (!fromVer)
            logger_1.logger.warn('updater', 'nothing to be updated');
        else {
            //开始升级
            let updated = false;
            for (let i = 0; i < versions.length; i++) {
                const { ver, program } = versions[i];
                //如果数据库中存在更高的版本则忽略
                if (dbVersions.some(dbv => dbv >= ver))
                    continue;
                //如果高于给定版本则忽略
                if (ver > dest)
                    continue;
                //否则进行升级
                await program(conn);
                //写入版本号
                await conn.exec('insert into _ver set ?', { ver, ctime: new Date() });
                updated = true;
            }
            if (updated)
                logger_1.logger.success('update', `update database to ${dest} successfully`);
            else
                logger_1.logger.warn('updater', 'nothing to be updated');
        }
        conn.close();
        conn = undefined;
    }
    catch (err) {
        logger_1.logger.error('update', err.message);
        if (err.sql)
            console.log(err.sql);
        if (conn)
            conn.close();
        throw err;
    }
}
exports.update = update;
