"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dbu = void 0;
const mysql_1 = __importDefault(require("mysql"));
const logger_1 = require("./logger");
var dbu;
(function (dbu) {
    /**
     * 生成键名称
     * @param column 列信息
     */
    function keyName(column) {
        if (typeof column == 'string')
            return column;
        return column.join('_');
    }
    /**
     * 将索引信息转换成SQL字符串
     * @param info 索引信息
     */
    function strIndex(info) {
        //索引名称
        const name = mysql_1.default.escapeId(keyName(info.columns));
        //索引类型
        const type = (info.type == 'index') ? 'index' : `${info.type} index`;
        //索引列
        const cols = info.columns.map(c => mysql_1.default.escapeId(c)).join(',');
        //生成字符串
        var str = `${type} ${name}(${cols})`;
        //ngram
        if (info.ngram && info.type == 'fulltext')
            str += ' with parser ngram';
        //完成
        return str;
    }
    /** 表格创建器 */
    class TableMaker {
        name;
        exec;
        comment;
        columns = [];
        indexes = [];
        links = [];
        primaries = [];
        constructor(name, exec, comment) {
            this.name = name;
            this.exec = exec;
            this.comment = comment;
        }
        /**
         * 添加一列
         * @param name 列名称
         * @param type 列类型
         * @param comment 列注释
         * @param length 列长度
         */
        column(name, type, option) {
            option = option || {};
            const required = (option.required === undefined) ? false : option.required;
            const dft = (() => {
                if (option.default === undefined)
                    return;
                if (typeof option.default == 'function')
                    return option.default() + '';
                return mysql_1.default.escape(option.default) + '';
            })();
            this.columns.push({
                name, type, length: (option.length === undefined) ? '' : (option.length + ''),
                required, inc: !!option.inc, default: dft, virtualGenerated: option.virtualGenerated,
                comment: option.comment || '',
            });
            return this;
        }
        /**
         * 添加索引
         * @param column 列名称
         * @param type 所有类型
         */
        index(column, type, option) {
            const c = (column instanceof Array) ? column : [column];
            this.indexes.push({ columns: c, type: type || 'index', ...option });
            return this;
        }
        /**
         * 添加外键
         * @param name 列名称
         * @param table 表名称
         * @param column 表中的列名称
         * @param option 选项
         */
        link(name, table, column, option) {
            option = option || {};
            this.links.push({ name, table, column, update: option.update || 'restrict', delete: option.delete || 'restrict' });
            return this;
        }
        /**
         * 定义主键
         * @param names 主键列名称
         */
        primary(...names) {
            this.primaries = names;
            return this;
        }
        /**
         * 定义大bingint类型的id列
         * @param option 列选项
         */
        id(option) {
            this.column('id', 'bigint', { required: true, inc: true, ...option });
            this.primary('id');
            return this;
        }
        /**
         * 定义uuid类型的id列
         * @param option 列选项
         */
        uuid(option) {
            this.column('id', 'varchar', { length: 255, required: true, ...option });
            this.primary('id');
            return this;
        }
        /**
         * 定义name列
         * @param option 列选项
         */
        nameColumn(option) {
            return this.column('name', 'varchar', { length: 255, required: true, ...option });
        }
        /**
         * 定义desc列
         * @param option 列选项
         */
        descColumn(option) {
            return this.column('desc', 'longtext', option);
        }
        /**
         * 定义排序字段
         */
        sortColumn() {
            return this.column('sort', 'bigint', { comment: '数据排序，一般在创建数据时填入ID即可' });
        }
        /**
         * 定义删除位
         */
        liveColumn() {
            return this.column('_live', 'tinyint', { required: true, default: 1, comment: '删除位，0表示删除' });
        }
        /**
         * 定义mkTime列
         */
        mkTimeColumn() {
            return this.column('mkTime', 'datetime', { default: () => `now()`, comment: '创建时间' });
        }
        /**
         * 定义upTime列
         * @param option 列选项
         */
        upTimeColumn() {
            return this.column('upTime', 'datetime', { default: () => `now()`, comment: '最后一次修改时间' });
        }
        /**
         * 定义20,2的定点小数列
         * @param name 列名称
         * @param option 列选项
         */
        decimalNormal(name, option) {
            return this.column(name, 'decimal', { length: '20,2', ...option });
        }
        /**
         * 定义ID类型(bigint)的外键
         * @param name 列名称
         * @param table 表名称
         * @param field 字段名称
         * @param option 外键选项
         */
        idLink(name, table, field, option) {
            const { update, delete: dlt, primary, ...columnOption } = option || {};
            this.column(name, 'bigint', columnOption);
            if (primary)
                this.primaries.push(name);
            this.link(name, table, field, {
                update: update,
                delete: dlt,
            });
            return this;
        }
        /**
         * 定义uuid类型的外键
         * @param name 列名称
         * @param table 表名称
         * @param field 字段名称
         * @param option 外键选项
         */
        uuidLink(name, table, field, option) {
            const { update, delete: dlt, primary, ...columnOption } = option || {};
            this.column(name, 'varchar', { length: 255, ...columnOption });
            if (primary)
                this.primaries.push(name);
            this.link(name, table, field, {
                update: update,
                delete: dlt,
            });
            return this;
        }
        /**
         * 定义完成
         */
        async done() {
            const mkcol = (col) => [
                mysql_1.default.escapeId(col.name),
                `${col.type}${col.length ? `(${col.length})` : ''}`,
                (col.required || this.primaries.includes(col.name)) ? `not null` : ``,
                col.inc ? 'auto_increment' : '',
                col.default ? `default ${col.default}` : '',
                col.virtualGenerated ? `generated always as (${col.virtualGenerated}) virtual` : '',
                col.comment ? `comment ${mysql_1.default.escape(col.comment)}` : ''
            ].filter(s => !!s).join(' ');
            const sql = `create table ${mysql_1.default.escapeId(this.name)} (${[
                ...this.columns.map(col => mkcol(col)),
                this.primaries ? `primary key (${this.primaries.map(p => mysql_1.default.escapeId(p)).join(',')})` : '',
                ...this.indexes.map(i => strIndex(i)),
            ].join(', ')})${this.comment ? ` comment=${mysql_1.default.escape(this.comment)}` : ''}`;
            logger_1.logger.info('update', `create table [${this.name}]`);
            await this.exec(sql);
            //等表创建完成之后，单独添加外键
            for (let i = 0; i < this.links.length; i++) {
                const link = this.links[i];
                const linkstr = [
                    `foreign key (${mysql_1.default.escapeId(link.name)}) references ${mysql_1.default.escapeId(link.table)}(${mysql_1.default.escapeId(link.column)})`,
                    `on update ${link.update}`,
                    `on delete ${link.delete}`,
                ].filter(s => !!s).join(' ');
                await this.exec(`alter table ${mysql_1.default.escapeId(this.name)} add ${linkstr}`);
            }
        }
    }
    /** 列修改器 */
    class ColumnUpdater {
        dbname;
        tableName;
        colname;
        exec;
        constructor(dbname, tableName, colname, exec) {
            this.dbname = dbname;
            this.tableName = tableName;
            this.colname = colname;
            this.exec = exec;
        }
        //获取列信息
        async columnInfo(name) {
            const getType = (t) => {
                const match = t.match(/^(\S+?)(\(([\s\S]+?)\))?$/);
                return {
                    type: match[1],
                    length: match[3],
                };
            };
            const [res] = await this.exec(`select * from information_schema.COLUMNS where TABLE_SCHEMA=${mysql_1.default.escape(this.dbname)} and TABLE_NAME=${mysql_1.default.escape(this.tableName)} and COLUMN_NAME=${mysql_1.default.escape(name)}`);
            const result = {
                name: res.COLUMN_NAME,
                ...getType(res.COLUMN_TYPE),
                comment: res.COLUMN_COMMENT,
                required: res.IS_NULLABLE == 'NO',
                inc: res.EXTRA == 'auto_increment',
                default: res.COLUMN_DEFAULT,
                virtualGenerated: res.EXTRA?.toLowerCase() == 'VIRTUAL GENERATED'.toLowerCase() ? res.GENERATION_EXPRESSION ?? undefined : undefined
            };
            return result;
        }
        //生成列SQL语句
        mkcol(col) {
            return [
                mysql_1.default.escapeId(col.name),
                `${col.type}${col.length ? `(${col.length})` : ''}`,
                (col.required) ? `not null` : ``,
                col.inc ? 'auto_increment' : '',
                col.default ? `default ${mysql_1.default.escape(col.default)}` : '',
                col.virtualGenerated ? `generated always as (${col.virtualGenerated}) virtual` : '',
                col.comment ? `comment ${mysql_1.default.escape(col.comment)}` : ''
            ].filter(s => !!s).join(' ');
        }
        /**
         * 重命名列
         * @param to 新名字
         */
        async rename(to) {
            const info = await this.columnInfo(this.colname);
            info.name = to;
            logger_1.logger.info('update', `alter column [${this.tableName}.${this.colname}] rename to [${to}]`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} change ${mysql_1.default.escapeId(this.colname)} ${this.mkcol(info)}`);
        }
        /**
         * 修改表类型
         * @param type 新的类型
         * @param length 类型的长度，有长度的必须指定
         */
        async type(type, length) {
            const info = await this.columnInfo(this.colname);
            info.type = type;
            info.length = length ? length + '' : '';
            logger_1.logger.info('update', `alter column [${this.tableName}.${this.colname}] type to ${type}`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} change ${mysql_1.default.escapeId(this.colname)} ${this.mkcol(info)}`);
        }
        /**
         * 修改自增信息
         * @param inc 是否自增
         */
        async inc(inc) {
            const info = await this.columnInfo(this.colname);
            info.inc = inc;
            logger_1.logger.info('update', `alter column [${this.tableName}.${this.colname}] increment to ${inc ? 'TRUE' : 'FALSE'}`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} change ${mysql_1.default.escapeId(this.colname)} ${this.mkcol(info)}`);
        }
        /**
         * 修改是否非空信息
         * @param req 是否非空
         */
        async required(req) {
            const info = await this.columnInfo(this.colname);
            info.required = req;
            logger_1.logger.info('update', `alter column [${this.tableName}.${this.colname}] ${req ? 'not null' : 'null'}`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} change ${mysql_1.default.escapeId(this.colname)} ${this.mkcol(info)}`);
        }
        /**
         * 修改默认值
         * @param defaultVal 默认值
         */
        async default(defaultVal) {
            const info = await this.columnInfo(this.colname);
            info.default = (() => {
                if (typeof defaultVal == 'function')
                    return defaultVal() + '';
                return defaultVal + '';
            })();
            logger_1.logger.info('update', `alter column [${this.tableName}.${this.colname}] default value to ${info.default}`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} change ${mysql_1.default.escapeId(this.colname)} ${this.mkcol(info)}`);
        }
        /**
         * 修改注释
         * @param comment 注释
         */
        async comment(comment) {
            const info = await this.columnInfo(this.colname);
            info.comment = comment;
            logger_1.logger.info('update', `alter column [${this.tableName}.${this.colname}] comment to ${comment}`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} change ${mysql_1.default.escapeId(this.colname)} ${this.mkcol(info)}`);
        }
        /**
         * 移动列到某列之后
         * @param column 列名称
         */
        async after(column) {
            const info = await this.columnInfo(this.colname);
            logger_1.logger.info('update', `alter column [${this.tableName}.${this.colname}] move after [${column}]`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} change ${mysql_1.default.escapeId(this.colname)} ${this.mkcol(info)} after ${mysql_1.default.escapeId(column)}`);
        }
        /**
         * 移动列到最开始
         */
        async first() {
            const info = await this.columnInfo(this.colname);
            logger_1.logger.info('update', `alter column [${this.tableName}.${this.colname}] move to first`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} change ${mysql_1.default.escapeId(this.colname)} ${this.mkcol(info)} first`);
        }
    }
    /** 表格修改器 */
    class TableUpdater {
        dbName;
        tableName;
        exec;
        constructor(dbName, tableName, exec) {
            this.dbName = dbName;
            this.tableName = tableName;
            this.exec = exec;
        }
        //获取外键信息
        async foreignKeyOf(column) {
            const sql = `select k.CONSTRAINT_NAME, k.TABLE_NAME, k.COLUMN_NAME ,k.REFERENCED_TABLE_NAME, k.REFERENCED_COLUMN_NAME, c.UPDATE_RULE, c.DELETE_RULE from information_schema.KEY_COLUMN_USAGE k join information_schema.REFERENTIAL_CONSTRAINTS c on c.CONSTRAINT_NAME=k.CONSTRAINT_NAME where k.TABLE_SCHEMA=${mysql_1.default.escape(this.dbName)} and k.TABLE_NAME=${mysql_1.default.escape(this.tableName)} and k.COLUMN_NAME=${mysql_1.default.escape(column)}`;
            const [key] = await this.exec(sql);
            // const [key] = keys.filter(key => key.REFERENCED_TABLE_SCHEMA && key.REFERENCED_TABLE_NAME && key.REFERENCED_COLUMN_NAME)
            if (!key)
                return null;
            return {
                name: key.CONSTRAINT_NAME,
                fromTable: key.TABLE_NAME,
                fromColumn: key.COLUMN_NAME,
                toTable: key.REFERENCED_TABLE_NAME,
                toColumn: key.REFERENCED_COLUMN_NAME,
                onUpdate: key.UPDATE_RULE,
                onDelete: key.DELETE_RULE,
            };
        }
        //获取主键列表
        async primaryKeys() {
            const keys = await this.keys();
            for (let i = 0; i < keys.length; i++) {
                if (keys[i].name == 'PRIMARY')
                    return keys[i].columns;
            }
            return [];
        }
        //获取索引列表
        async keys() {
            const sql = `select * from information_schema.STATISTICS where TABLE_SCHEMA=${mysql_1.default.escape(this.dbName)} and TABLE_NAME=${mysql_1.default.escape(this.tableName)}`;
            const keys = await this.exec(sql);
            const buffer = [];
            keys.forEach(key => {
                for (let i = 0; i < buffer.length; i++) {
                    if (buffer[i].name == key.INDEX_NAME) {
                        buffer[i].columns.push(key.COLUMN_NAME);
                        return;
                    }
                }
                buffer.push({
                    name: key.INDEX_NAME,
                    columns: [key.COLUMN_NAME]
                });
            });
            return buffer;
        }
        /**
         * 修改表注释
         * @param comment 表注释
         */
        async comment(comment) {
            logger_1.logger.info('update', `alter table [${this.tableName}] set comment`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} comment ${mysql_1.default.escape(comment)}`);
        }
        /**
         * 重命名表
         * @param to 新名字
         */
        async rename(to) {
            logger_1.logger.info('update', `rename table [${this.tableName}] to [${to}]`);
            await this.exec(`rename table ${mysql_1.default.escapeId(this.tableName)} to ${mysql_1.default.escapeId(to)}`);
        }
        /**
         * 设置主键
         * @param names 主键名称
         */
        async primary(...names) {
            if (names.length)
                logger_1.logger.info('update', `alter table [${this.tableName}] set primary (${names.join(',')})`);
            else
                logger_1.logger.info('update', `alter table [${this.tableName}] drop primary`);
            //获取主键列表
            const keys = await this.primaryKeys();
            if (keys.length)
                await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} drop primary key`);
            if (names.length)
                await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} add primary key (${names.map(n => mysql_1.default.escapeId(n)).join(',')})`);
        }
        /**
         * 添加列
         * @param name 列名称
         * @param type 列类型
         * @param option 列选项
         */
        async addColumn(name, type, option) {
            const { length = '', required = false, inc = false, comment = undefined, default: dftv = undefined, virtualGenerated = undefined } = option || {};
            const dft = (() => {
                if (dftv === undefined)
                    return;
                if (typeof dftv === 'function')
                    return dftv() + '';
                return mysql_1.default.escape(dftv);
            })();
            const mkcol = () => [
                mysql_1.default.escapeId(name),
                `${type}${length ? `(${length})` : ''}`,
                required ? `not null` : ``,
                inc ? 'auto_increment' : '',
                dft ? `default ${dft}` : '',
                virtualGenerated ? `generated always as (${virtualGenerated}) virtual` : '',
                comment ? `comment ${mysql_1.default.escape(comment)}` : ''
            ].filter(s => !!s).join(' ');
            logger_1.logger.info('update', `alter table [${this.tableName}] add column [${name}]`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} add column ${mkcol()}`);
        }
        /**
         * 删除列
         * @param name 列名称
         */
        async dropColumn(name) {
            logger_1.logger.info('update', `alter table [${this.tableName}] drop column [${name}]`);
            await this.dropLink(name);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} drop column ${mysql_1.default.escapeId(name)}`);
        }
        /**
         * 修改列
         * @param name 列名称
         */
        column(name) {
            return new ColumnUpdater(this.dbName, this.tableName, name, this.exec);
        }
        /**
         * 删除索引
         * @param columns 索引列名称
         */
        async dropKey(columns) {
            logger_1.logger.info('update', `alter table [${this.tableName}] drop index of [${columns}]`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} drop index ${keyName(columns)}`);
        }
        /**
         * 添加索引
         * @param columns 列名称
         * @param type 索引类型
         * @param option 索引选项
         */
        async addIndex(columns, type = 'index', option) {
            columns = (columns instanceof Array) ? columns : [columns];
            //检测索引是否存在，如果存在则删除
            const kname = keyName(columns);
            const keys = await this.keys();
            if (keys.some(key => key.name == kname))
                await this.dropKey(columns);
            //添加索引
            logger_1.logger.info('update', `alter table [${this.tableName}] add index of [${columns}]`);
            const indexStr = strIndex({ columns, type, ...option });
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} add ${indexStr}`);
        }
        /**
         * 删除给定列的外键
         * @param column 列名称
         */
        async dropLink(column) {
            const key = await this.foreignKeyOf(column);
            if (!key)
                return;
            logger_1.logger.info('update', `alter table [${this.tableName}] drop foreign key of [${column}]`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} drop foreign key ${mysql_1.default.escapeId(key.name)}`);
        }
        /**
         * 添加外键
         * @param name 要设置外键的字段名称
         * @param table 目标表名称
         * @param column 目标表列名称
         * @param option 外键选项
         */
        async addLink(name, table, column, option) {
            option = option || {};
            const ref = [
                option.update ? `on update ${option.update}` : '',
                option.delete ? `on delete ${option.delete}` : '',
            ].filter(s => !!s).join(' ');
            logger_1.logger.info('update', `alter table [${this.tableName}] add foreign key of [${column}]`);
            await this.exec(`alter table ${mysql_1.default.escapeId(this.tableName)} add foreign key (${mysql_1.default.escapeId(name)}) references ${mysql_1.default.escapeId(table)}(${mysql_1.default.escapeId(column)}) ${ref}`);
        }
        /**
         * 设置外键（如果存在则先删除）
         * @param name 要设置外键的字段名称
         * @param table 目标表名称
         * @param column 目标表列名称
         * @param option 外键选项
         */
        async link(name, table, column, option) {
            //删除外键
            await this.dropLink(name);
            //添加外键
            await this.addLink(name, table, column, option);
        }
    }
    /** MySQL连接 */
    class MyConnection {
        conn;
        dbname;
        constructor(option) {
            this.conn = mysql_1.default.createConnection({
                host: option.host || 'localhost',
                port: option.port || 3306,
                user: option.username,
                password: option.password,
                charset: option.charset || 'utf8mb4'
            });
        }
        /**
         * 连接数据库
         */
        async connect() {
            return new Promise((resolve, reject) => this.conn.connect((err, res) => err ? reject(err) : resolve()));
        }
        /**
         * 关闭连接
         */
        close() {
            this.conn.end();
        }
        /**
         * 使用给第数据库
         * @param dbName 数据库名称
         */
        async use(dbName) {
            this.dbname = dbName;
            await this.exec('use ??', dbName);
        }
        /**
         * 执行sql查询语句
         * @param sql SQL语句
         * @param args 参数
         */
        async query(sql, ...args) {
            return new Promise((resolve, reject) => this.conn.query(sql, args, (err, res) => err ? reject(err) : resolve(res)));
        }
        /**
         * 执行SQL修改语句
         * @param sql SQL语句
         * @param args 参数
         */
        async exec(sql, ...args) {
            return new Promise((resolve, reject) => this.conn.query(sql, args, (err, res) => err ? reject(err) : resolve(res)));
        }
        /**
         * 创建表
         * @param name 表名称
         * @param comment 表注释
         */
        mktbl(name, comment) {
            return new TableMaker(name, this.exec.bind(this), comment);
        }
        /**
         * 修改表信息
         * @param name 表名称
         */
        uptbl(name) {
            if (!this.dbname)
                throw new Error('unknown database name when update table');
            return new TableUpdater(this.dbname, name, this.exec.bind(this));
        }
        /**
         * 插入数据
         * @param value 数据值
         */
        async insert(table, value) {
            return await this.exec('insert into ?? set ?', table, value);
        }
        /**
         * 插入多条数据
         * @param values 数据值
         */
        async insertMany(table, values) {
            const result = [];
            for (let i = 0; i < values.length; i++) {
                const resi = await this.insert(table, values[i]);
                result.push(resi.insertId);
                logger_1.logger.logs(`Gcomplate: ${parseInt((i + 1) * 100 / values.length)}% (${i + 1}/${values.length})`);
            }
            return result;
        }
        /**
         * 数据修改
         * @param table 表格
         * @param key 主键列，可以是多个
         * @param columns 要查询的列
         * @param updater 更改器，这里面返回新的数据，如果返回undefined则不处理
         */
        async updata(table, key, columns, updater) {
            logger_1.logger.info('update', `updating ${table} for columns of ${columns.join(',')}`);
            //每次处理的数据量
            const everytime = 200;
            //此函数用于更新部分数据，返回更新的数据量
            const updateSome = async (limit) => {
                const keys = (key instanceof Array) ? key : [key];
                const datas = await this.query(`select ${[...keys, ...columns].map(c => mysql_1.default.escapeId(c)).join(',')} from ${mysql_1.default.escapeId(table)} limit ${limit}, ${everytime}`);
                for (let i = 0; i < datas.length; i++) {
                    const result = await updater(datas[i]);
                    if (result === undefined)
                        continue;
                    //生成where条件
                    const whereStr = [];
                    const whereParams = [];
                    keys.forEach(key => {
                        whereStr.push('??=?');
                        whereParams.push(key, datas[i][key]);
                    });
                    //执行
                    await this.exec('update ?? set ? where ' + whereStr.join(' and '), table, result, ...whereParams);
                }
                return datas.length;
            };
            //开始更新，如果没有完成则一直更新
            let updated = 0;
            while (true) {
                const count = await updateSome(updated);
                updated += count;
                if (count < everytime)
                    break;
            }
            return updated;
        }
    }
    /**
     * 连接到数据库
     * @param conf 连接配置
     */
    function getConnection(conf) {
        return new MyConnection(conf);
    }
    dbu.getConnection = getConnection;
    const versions = [];
    /**
     * 添加一个版本
     * @param ver 版本号，使用浮点数（小数位占两位），例如3.02
     * @param program 版本升级程序
     */
    function version(ver, program) {
        versions.push({ ver, program });
    }
    dbu.version = version;
    /**
     * 获取版本列表
     */
    function getVersions() {
        return versions;
    }
    dbu.getVersions = getVersions;
})(dbu = exports.dbu || (exports.dbu = {}));
