export declare namespace dbu {
    /** 数据库连接选项 */
    export interface IMysqlConnOption {
        /** 主机名，默认localhost */
        host?: string;
        /** 端口，默认3306 */
        port?: number;
        /** 用户名 */
        username?: string;
        /** 用户密码 */
        password?: string;
        /** 字符集，默认utf8mb4 */
        charset?: string;
    }
    /** 数据库列类型 */
    type IColumnType = 'int' | 'bigint' | 'tinyint' | 'varchar' | 'char' | 'text' | 'longtext' | 'date' | 'time' | 'datetime' | 'timestamp' | 'json' | 'enum' | 'decimal' | 'tinyblob' | 'blob' | 'mediumblob' | 'longblob';
    /** 列选项 */
    interface IColumnOption {
        /** 是否非空 */
        required?: boolean;
        /** 是否自增 */
        inc?: boolean;
        /** 注释 */
        comment?: string;
        /** 类型长度 */
        length?: string | number;
        /** 默认值 */
        default?: string | number | boolean | (() => string | number | boolean);
        /** 虚拟列 ( GENERATED ALWAYS AS (substring(name,2,1)) virtual ) */
        virtualGenerated?: string;
    }
    /** 索引信息 */
    interface IIndexInfo {
        /** 类型 */
        type: 'index' | 'unique' | 'fulltext';
        /** 列名称 */
        columns: Array<string>;
        /** 是否使用ngram分词器，在全文索引时可以设置 */
        ngram?: boolean;
    }
    /** 外键级联选项 */
    interface ILinkRefOption {
        /** 更新选项 */
        update: 'restrict' | 'cascade' | 'set null' | 'no action' | 'set default';
        /** 删除选项 */
        delete: 'restrict' | 'cascade' | 'set null' | 'no action' | 'set default';
    }
    /** 表格创建器 */
    class TableMaker {
        private name;
        private exec;
        private comment?;
        private columns;
        private indexes;
        private links;
        private primaries;
        constructor(name: string, exec: (sql: string) => any, comment?: string | undefined);
        /**
         * 添加一列
         * @param name 列名称
         * @param type 列类型
         * @param comment 列注释
         * @param length 列长度
         */
        column(name: string, type: IColumnType, option?: IColumnOption): this;
        /**
         * 添加索引
         * @param column 列名称
         * @param type 所有类型
         */
        index(column: string | Array<string>, type?: IIndexInfo['type'], option?: Pick<IIndexInfo, 'ngram'>): this;
        /**
         * 添加外键
         * @param name 列名称
         * @param table 表名称
         * @param column 表中的列名称
         * @param option 选项
         */
        link(name: string, table: string, column: string, option?: Partial<ILinkRefOption>): this;
        /**
         * 定义主键
         * @param names 主键列名称
         */
        primary(...names: Array<string>): this;
        /**
         * 定义大bingint类型的id列
         * @param option 列选项
         */
        id(option?: IColumnOption): this;
        /**
         * 定义uuid类型的id列
         * @param option 列选项
         */
        uuid(option?: IColumnOption): this;
        /**
         * 定义name列
         * @param option 列选项
         */
        nameColumn(option?: IColumnOption): this;
        /**
         * 定义desc列
         * @param option 列选项
         */
        descColumn(option?: IColumnOption): this;
        /**
         * 定义排序字段
         */
        sortColumn(): this;
        /**
         * 定义删除位
         */
        liveColumn(): this;
        /**
         * 定义mkTime列
         */
        mkTimeColumn(): this;
        /**
         * 定义upTime列
         * @param option 列选项
         */
        upTimeColumn(): this;
        /**
         * 定义20,2的定点小数列
         * @param name 列名称
         * @param option 列选项
         */
        decimalNormal(name: string, option?: IColumnOption): this;
        /**
         * 定义ID类型(bigint)的外键
         * @param name 列名称
         * @param table 表名称
         * @param field 字段名称
         * @param option 外键选项
         */
        idLink(name: string, table: string, field: string, option?: IColumnOption & Partial<ILinkRefOption> & {
            primary?: boolean;
        }): this;
        /**
         * 定义uuid类型的外键
         * @param name 列名称
         * @param table 表名称
         * @param field 字段名称
         * @param option 外键选项
         */
        uuidLink(name: string, table: string, field: string, option?: IColumnOption & Partial<ILinkRefOption> & {
            primary?: boolean;
        }): this;
        /**
         * 定义完成
         */
        done(): Promise<void>;
    }
    /** 列修改器 */
    class ColumnUpdater {
        private dbname;
        private tableName;
        private colname;
        private exec;
        constructor(dbname: string, tableName: string, colname: string, exec: (sql: string) => any);
        private columnInfo;
        private mkcol;
        /**
         * 重命名列
         * @param to 新名字
         */
        rename(to: string): Promise<void>;
        /**
         * 修改表类型
         * @param type 新的类型
         * @param length 类型的长度，有长度的必须指定
         */
        type(type: IColumnType, length?: string | number): Promise<void>;
        /**
         * 修改自增信息
         * @param inc 是否自增
         */
        inc(inc: boolean): Promise<void>;
        /**
         * 修改是否非空信息
         * @param req 是否非空
         */
        required(req: boolean): Promise<void>;
        /**
         * 修改默认值
         * @param defaultVal 默认值
         */
        default(defaultVal: IColumnOption['default']): Promise<void>;
        /**
         * 修改注释
         * @param comment 注释
         */
        comment(comment: string): Promise<void>;
        /**
         * 移动列到某列之后
         * @param column 列名称
         */
        after(column: string): Promise<void>;
        /**
         * 移动列到最开始
         */
        first(): Promise<void>;
    }
    /** 表格修改器 */
    class TableUpdater {
        private dbName;
        private tableName;
        private exec;
        constructor(dbName: string, tableName: string, exec: (sql: string) => any);
        private foreignKeyOf;
        private primaryKeys;
        private keys;
        /**
         * 修改表注释
         * @param comment 表注释
         */
        comment(comment: string): Promise<void>;
        /**
         * 重命名表
         * @param to 新名字
         */
        rename(to: string): Promise<void>;
        /**
         * 设置主键
         * @param names 主键名称
         */
        primary(...names: Array<string>): Promise<void>;
        /**
         * 添加列
         * @param name 列名称
         * @param type 列类型
         * @param option 列选项
         */
        addColumn(name: string, type: IColumnType, option?: IColumnOption): Promise<void>;
        /**
         * 删除列
         * @param name 列名称
         */
        dropColumn(name: string): Promise<void>;
        /**
         * 修改列
         * @param name 列名称
         */
        column(name: string): ColumnUpdater;
        /**
         * 删除索引
         * @param columns 索引列名称
         */
        dropKey(columns: Array<string> | string): Promise<void>;
        /**
         * 添加索引
         * @param columns 列名称
         * @param type 索引类型
         * @param option 索引选项
         */
        addIndex(columns: Array<string> | string, type?: IIndexInfo['type'], option?: Pick<IIndexInfo, 'ngram'>): Promise<void>;
        /**
         * 删除给定列的外键
         * @param column 列名称
         */
        dropLink(column: string): Promise<void>;
        /**
         * 添加外键
         * @param name 要设置外键的字段名称
         * @param table 目标表名称
         * @param column 目标表列名称
         * @param option 外键选项
         */
        addLink(name: string, table: string, column: string, option?: Partial<ILinkRefOption>): Promise<void>;
        /**
         * 设置外键（如果存在则先删除）
         * @param name 要设置外键的字段名称
         * @param table 目标表名称
         * @param column 目标表列名称
         * @param option 外键选项
         */
        link(name: string, table: string, column: string, option?: Partial<ILinkRefOption>): Promise<void>;
    }
    /** MySQL连接 */
    class MyConnection {
        private conn;
        private dbname?;
        constructor(option: IMysqlConnOption);
        /**
         * 连接数据库
         */
        connect(): Promise<void>;
        /**
         * 关闭连接
         */
        close(): void;
        /**
         * 使用给第数据库
         * @param dbName 数据库名称
         */
        use(dbName: string): Promise<void>;
        /**
         * 执行sql查询语句
         * @param sql SQL语句
         * @param args 参数
         */
        query<T = any>(sql: string, ...args: Array<any>): Promise<Array<T>>;
        /**
         * 执行SQL修改语句
         * @param sql SQL语句
         * @param args 参数
         */
        exec(sql: string, ...args: Array<any>): Promise<any>;
        /**
         * 创建表
         * @param name 表名称
         * @param comment 表注释
         */
        mktbl(name: string, comment?: string): TableMaker;
        /**
         * 修改表信息
         * @param name 表名称
         */
        uptbl(name: string): TableUpdater;
        /**
         * 插入数据
         * @param value 数据值
         */
        insert(table: string, value: {
            [i: string]: any;
        }): Promise<{
            insertId: number | string;
        }>;
        /**
         * 插入多条数据
         * @param values 数据值
         */
        insertMany(table: string, values: Array<{
            [i: string]: any;
        }>): Promise<Array<number | string>>;
        /**
         * 数据修改
         * @param table 表格
         * @param key 主键列，可以是多个
         * @param columns 要查询的列
         * @param updater 更改器，这里面返回新的数据，如果返回undefined则不处理
         */
        updata<T = any>(table: string, key: string | Array<string>, columns: Array<string>, updater: (oldData: T) => any): Promise<number>;
    }
    /**
     * 连接到数据库
     * @param conf 连接配置
     */
    export function getConnection(conf: IMysqlConnOption): MyConnection;
    /**
     * 添加一个版本
     * @param ver 版本号，使用浮点数（小数位占两位），例如3.02
     * @param program 版本升级程序
     */
    export function version(ver: number, program: (conn: MyConnection) => Promise<void>): void;
    /**
     * 获取版本列表
     */
    export function getVersions(): {
        ver: number;
        program: (conn: MyConnection) => Promise<void>;
    }[];
    export {};
}
